/*
 *  Cage.java
 *  Copyright Simon Hern 2009
 *  Contact: dishmoth@yahoo.co.uk, www.dishmoth.com
 */

package Space;

import java.util.LinkedList;
import javax.media.opengl.*;

// barrier at the edge of a planetary system
public class Cage extends Sprite {

  // bounding planes (normals point inwards)
  private Plane mPlanes[];

  // scale of the cage (distance of vertices from origin)
  private float mCageRadius;
  
  // scale of the cage (minimum distance of face planes from origin)
  private float mMinimumRadius;
  
  // prepare resources
  static public void init(GL2 gl) {

    CageGL.init(gl);
    
  } // init()
  
  // discard any resources
  static public void dispose(GL2 gl) {
    
    CageGL.dispose(gl);
    
  } // dispose()
  
  // constructor
  public Cage(float radius) {
    
    assert( radius > 0.0 );
    mCageRadius = radius;

    mPlanes = Icosahedron.getPlanes(mCageRadius);

    mMinimumRadius = Math.abs( mPlanes[0].offset() );
    
  } // constructor

  // access to the bounding planes of the cage
  public Plane[] getPlanes() { return mPlanes; }
  
  // position (centre of bounding sphere) (not applicable here)
  @Override public float getXPos() { return 0.0f; }
  @Override public float getYPos() { return 0.0f; }
  @Override public float getZPos() { return 0.0f; }

  // radii of bounding spheres (not applicable here)
  @Override public float getCollisionRadius() { return 0.0f; }
  @Override public float getVisibleRadius() { return 0.0f; }
  @Override public float getTargetRadius() { return 0.0f; }
  
  // minimum and maximum distance of cage from origin
  public float getMaxRadius() { return mCageRadius; }
  public float getMinRadius() { return mMinimumRadius; }
  
  // change position, etc. (nothing needs doing here)
  @Override 
  public void advance(float dt, LinkedList<Sprite> sprites) {
  } // Sprite.advance()
  
  // draw the object (delegate to CageGL)
  @Override 
  public void display(GL2 gl) {
    
    gl.glPushMatrix();
    gl.glScalef(mCageRadius, mCageRadius, mCageRadius);
    CageGL.display(gl);
    gl.glPopMatrix();
    
  } // Sprite.display()

} // class Cage
