/*
 *  Plane.java
 *  Copyright Simon Hern 2009
 *  Contact: dishmoth@yahoo.co.uk, www.dishmoth.com
 */

package Space;

// simple 3D plane (defined by "normal . point = offset")
public class Plane {

  // unit normal vector
  private float mNormal[];
  
  // distance of the plane from the origin
  private float mOffset;
  
  // constructor
  public Plane(float nx, float ny, float nz, float offset) {
    
    final float d2 = nx*nx + ny*ny + nz*nz;
    assert( d2 > 1.0e-6 );
    final float d = (float)Math.sqrt(d2);
    
    mNormal = new float[]{ nx/d, ny/d, nz/d };
    mOffset = offset;
    
  } // constructor
  
  // access to unit normal vector
  public float[] normal() { return mNormal; }
  
  // access to offset distance
  public float offset() { return mOffset; }
  
  // distance of point from plane (positive infront, negative behind)
  public float distance(float x, float y, float z) {
    
    final float dot = x*mNormal[0] + y*mNormal[1] + z*mNormal[2];
    return ( dot - mOffset );
    
  } // distance()
  
} // Plane
