/**
 *  Col24.java
 *  Copyright Simon Hern 2009
 *  Contact: dishmoth@yahoo.co.uk, www.dishmoth.com
 */

package Space.patterns;

import java.awt.Color;

// simple utilities for handling 24-bit colour values
public class Col24 {

  // combine three 8-bit integer components into a 24-bit integer
  static public int rgb(int red, int green, int blue) {
    
    assert( red >= 0   && red <= 255 );
    assert( green >= 0 && green <= 255 );
    assert( blue >= 0  && blue <= 255 );
    
    return ( (red<<16) | (green<<8) | blue );
    
  } // rgb(int)
  
  // combine three floating-point components into a 24-bit integer
  static public int rgb(float red, float green, float blue) {
    
    assert( red >= 0.0f   && red <= 1.0f );
    assert( green >= 0.0f && green <= 1.0f );
    assert( blue >= 0.0f  && blue <= 1.0f );
    
    final int r = (int)Math.round(255*red),
              g = (int)Math.round(255*green),
              b = (int)Math.round(255*blue);
    
    return ( (r<<16) | (g<<8) | b );
    
  } // rgb(int)
  
  // convert a Java Color into 24-bit integer format
  static public int rgb(Color c) {
    
    return c.getRGB();
    
  } // rgb(Color)

  // extract 8-bit colour components
  static public int red(int rgb)   { return ( (rgb>>16) & 0xFF ); }
  static public int green(int rgb) { return ( (rgb>>8) & 0xFF ); }
  static public int blue(int rgb)  { return ( rgb & 0xFF ); }
  
  // define a 24-bit colour through HSB values 
  static public int hsb(float hue, float saturation, float brightness) {
    
    return Color.HSBtoRGB(hue, saturation, brightness);
    
  } // hsb()
  
  // interpolate between first colour (h=0.0) and second colour (h=1.0)
  static public int interp(int rgb0, int rgb1, float h) {
    
    assert( rgb0 >= 0 && rgb0 < (1<<24) );
    assert( rgb1 >= 0 && rgb1 < (1<<24) );
    
    if ( h <= 0.0f ) return rgb0;
    if ( h >= 1.0f ) return rgb1;
    
    final int r0 = (rgb0 >> 16) & 0xFF,
              g0 = (rgb0 >> 8) & 0xFF,
              b0 = rgb0 & 0xFF;
    final int r1 = (rgb1 >> 16) & 0xFF,
              g1 = (rgb1 >> 8) & 0xFF,
              b1 = rgb1 & 0xFF;
    
    final int r = r0 + (int)Math.round((r1-r0)*h),
              g = g0 + (int)Math.round((g1-g0)*h),
              b = b0 + (int)Math.round((b1-b0)*h);
    
    return ( (r<<16) | (g<<8) | b );
    
  } // interp()
  
} // class Col24
