/**
 *  ColourMap.java
 *  Copyright Simon Hern 2009
 *  Contact: dishmoth@yahoo.co.uk, www.dishmoth.com
 */

package Space.patterns;

// a one-dimensional range of colours (linearly interpolated)
public class ColourMap {

  // sample points (monotonically increasing)
  private float mX[];
  
  // sample colours (24-bit integer format)
  private int mRGB[];
  
  // constructor
  public ColourMap(float x[], int rgb[]) {
    
    assert( x != null );
    assert( rgb != null );
    assert( x.length == rgb.length );
    
    for ( int k = 0 ; k < x.length-1 ; k++ ) {
      assert( x[k] < x[k+1] );
    }
    for ( int k = 0 ; k < rgb.length ; k++ ) {
      assert( rgb[k] >= 0 && rgb[k] < (1<<24) ); 
    }
    
    mX = x;
    mRGB = rgb;
    
  } // constructor
  
  // interpolate between the colour samples
  public int rgb(float x) {
    
    if ( x <= mX[0] ) return mRGB[0];
    for ( int k = 0 ; k < mX.length-1 ; k++ ) {
      assert( x > mX[k] );
      if ( x <= mX[k+1] ) {
        return Col24.interp(mRGB[k], mRGB[k+1], (x-mX[k])/(mX[k+1]-mX[k]));
      }
    }
    assert( x > mX[ mX.length-1 ] );
    return mRGB[ mRGB.length-1 ];
    
  } // rgb()
  
} // class ColourMap
