/**
 *  Normalize.java
 *  Copyright Simon Hern 2009
 *  Contact: dishmoth@yahoo.co.uk, www.dishmoth.com
 */

package Space.patterns;

// rescale and clip the output of a function to the range [0,1]
public class Normalize extends Function {

  // the underlying function
  private Function mFunc;
  
  // range of function output to normalize
  private float mMin,
                mMax;
  
  // constructor, map [-min,+max] to [0,1]
  public Normalize(float min, float max, Function func) {
    
    assert( func != null );
    mFunc = func;
    
    assert( min < max );
    mMin = min;
    mMax = max;
    
  } // constructor

  // map the function output to [0,1], clipping if necessary 
  @Override
  public float value(float x, float y, float z) {
    
    float v = mFunc.value(x, y, z);
    if ( v <= mMin ) return 0.0f;
    if ( v >= mMax ) return 1.0f;
    return (v - mMin)/(mMax - mMin);
    
  } // Function.value(x,y,z)
  
} // class Normalize
