/**
 *  Scale.java
 *  Copyright Simon Hern 2009
 *  Contact: dishmoth@yahoo.co.uk, www.dishmoth.com
 */

package Space.patterns;

// rescale the xyz-coordinates going into a function evaluation
public class Scale extends Function {

  // the underlying function
  private Function mFunc;
  
  // scale factors
  private float mScaleX,
                mScaleY,
                mScaleZ;
  
  // constructor, input coordinates are scaled equally
  public Scale(float scale, Function func) {
    
    assert( func != null );
    mFunc = func;
    
    mScaleX = mScaleY = mScaleZ = scale;
    
  } // constructor(scale)

  // constructor, input coordinates are scaled individually
  public Scale(float xScale, float yScale, float zScale, Function func) {
    
    assert( func != null );
    mFunc = func;
    
    mScaleX = xScale;
    mScaleY = yScale;
    mScaleZ = zScale;
    
  } // constructor(x,y,z)

  // evaluate the function at a scaled xyz-position
  @Override
  public float value(float x, float y, float z) {
    
    return mFunc.value(x*mScaleX, y*mScaleY, z*mScaleZ);
    
  } // Function.value(x,y,z)
  
} // class Scale
