/**
 *  Turbulence.java
 *  Copyright Simon Hern 2009
 *  Contact: dishmoth@yahoo.co.uk, www.dishmoth.com
 */

package Space.patterns;

// turbulence function (sum of absolute noise functions)
public class Turbulence extends Function {

  // the underlying noise function
  private Function mNoise = null;

  // number of octaves in the noise summation
  private int mNumOctaves;
  
  // reduction factor applied to higher octaves
  private float mOctaveDecay;
  
  // constructor
  public Turbulence(Function baseNoise, int numOctaves) {

    assert( baseNoise != null );
    mNoise = baseNoise;
    
    assert( numOctaves >= 1 );
    mNumOctaves = 3;
    
    mOctaveDecay = 0.5f;

  } // constructor
  
  // constructor
  public Turbulence(Function baseNoise, int numOctaves, float octaveDecay) {

    assert( baseNoise != null );
    mNoise = baseNoise;
    
    assert( numOctaves >= 1 );
    mNumOctaves = numOctaves;
    
    assert( octaveDecay > 0.0f && octaveDecay < 1.0f );
    mOctaveDecay = octaveDecay;

  } // constructor
  
  // evaluate the noise value at a three-dimensional position
  @Override
  public float value(float x, float y, float z) {
    
    float val = Math.abs(mNoise.value(x, y, z));
    
    float scale  = 2.0f,
          factor = mOctaveDecay;
    for ( int k = 1 ; k < mNumOctaves ; k++ ) {
      val += factor*Math.abs(mNoise.value(scale*x, scale*y, scale*z));
      scale *= 2.0f;
      factor *= mOctaveDecay;
    }

    return val;
    
  } // Function.value(x,y,z)

} // class Turbulence
