/**
 *  CellularSkin.java
 *  Copyright Simon Hern 2009
 *  Contact: dishmoth@yahoo.co.uk, www.dishmoth.com
 */

package Space.planets;

import Space.patterns.*;

// a texture for a planet
public class CellularSkin implements SphericalSkin {

  // noise functions
  private Function mCellNoise,
                   mEdgeNoise;

  // colour ranges
  private ColourMap mEdgeColour;

  // fixed colour
  private int mCellColour;
  
  // constructor
  public CellularSkin() {
    
    mCellNoise = new Normalize(0.0f, 0.08f, 
                   new VoronoiSphere(88, 200));
    mEdgeNoise = new Scale(2.0f, 
                   new Perlin3D(99));
    
    float x[] = { -0.5f, 
                  +0.5f };
    int rgb[] = { Col24.rgb(0.0f, 0.5f, 0.0f), 
                  Col24.rgb(0.0f, 0.2f, 0.0f) };
    mEdgeColour = new ColourMap(x, rgb);

    mCellColour = Col24.rgb(0.9f, 0.9f, 0.9f);

  } // constructor
  
  // return a 24-bit RGB colour for a point on the unit sphere
  @Override
  public int colour(float x, float y, float z) {

    float fc = mCellNoise.value(x, y, z),
          fe = mEdgeNoise.value(x, y, z);
    
    int rgb = Col24.interp(mCellColour, mEdgeColour.rgb(fe), fc);

    return rgb;
    
  } // SphericalSkin.colour()

} // class CellularSkin
