/**
 *  CloudySkin.java
 *  Copyright Simon Hern 2009
 *  Contact: dishmoth@yahoo.co.uk, www.dishmoth.com
 */

package Space.planets;

import Space.patterns.*;

// a texture for a planet
public class CloudySkin implements SphericalSkin {

  // noise functions
  private Function mCloudNoise,
                   mSeaNoise,
                   mPatternNoise;

  // colour ranges
  private ColourMap mCloudColour,
                    mSeaColour;

  // constructor
  public CloudySkin() {
    
    mCloudNoise = new Scale(4.0f,
                    new FractalSum(new Perlin3D(1), 2));
    
    mSeaNoise = new Scale(3.0f, 
                  new FractalSum(new Perlin3D(2), 2));

    mPatternNoise = new Normalize(-0.3f, +0.5f,
                      new Scale(1.0f, 3.0f, 1.0f,
                        new FractalSum(new Perlin3D(6), 3)));
    
    float x1[] = { -1.5f, 
                   +1.5f };
    int rgb1[] = { Col24.rgb(0.5f, 0.3f, 0.2f), 
                   Col24.rgb(1.0f, 0.5f, 0.0f) };
    mCloudColour = new ColourMap(x1, rgb1);

    float x2[] = { -0.2f, 
                    0.0f,
                   +0.2f };
    int rgb2[] = { Col24.rgb(0.0f, 0.0f, 0.35f), 
                   Col24.rgb(0.05f,0.05f,0.4f), 
                   Col24.rgb(0.0f, 0.0f, 0.35f) };
    mSeaColour = new ColourMap(x2, rgb2);

  } // constructor
  
  // return a 24-bit RGB colour for a point on the unit sphere
  @Override
  public int colour(float x, float y, float z) {

    float fc = mCloudNoise.value(x, y, z);
    int rgbCloud = mCloudColour.rgb(fc);
    
    float fs = mSeaNoise.value(x, y, z);
    int rgbSea = mSeaColour.rgb(fs);
    
    float fp = mPatternNoise.value(x, y, z);
    int rgb = Col24.interp(rgbSea, rgbCloud, fp);
    
    return rgb;
    
  } // SphericalSkin.colour()

} // class CloudySkin
