/**
 *  FieryOrangeSkin.java
 *  Copyright Simon Hern 2009
 *  Contact: dishmoth@yahoo.co.uk, www.dishmoth.com
 */

package Space.planets;

import Space.patterns.*;

// a texture for a planet
public class FieryOrangeSkin implements SphericalSkin {

  // noise functions
  private Function mPatternNoise,
                   mColourNoise;

  // colour ranges
  private ColourMap mColour1,
                    mColour2;
  
  // constructor
  public FieryOrangeSkin() {
    
    mPatternNoise = new Normalize(0.1f, 0.6f,
                      new Scale(1.5f, 3.0f, 1.5f,
                        new Turbulence(new Perlin3D(0), 3)));
    
    mColourNoise = new Normalize(-0.5f, +0.5f, 
                     new Scale(2.0f,
                       new Perlin3D(1)));
    
    float x[] = { 0.0f, 
                  0.5f, 
                  1.0f };
    int rgb1[] = { Col24.rgb(1.0f, 0.0f, 0.0f), 
                   Col24.rgb(0.5f, 0.0f, 0.0f),
                   Col24.rgb(0.0f, 0.0f, 0.0f) },
        rgb2[] = { Col24.rgb(1.0f, 0.5f, 0.0f), 
                   Col24.rgb(0.5f, 0.0f, 0.0f),
                   Col24.rgb(0.0f, 0.0f, 0.0f) };
    
    mColour1 = new ColourMap(x, rgb1);
    mColour2 = new ColourMap(x, rgb2);
    
  } // constructor
  
  // return a 24-bit RGB colour for a point on the unit sphere
  @Override
  public int colour(float x, float y, float z) {
    
    float fp = mPatternNoise.value(x, y, z);

    int rgb1 = mColour1.rgb(fp),
        rgb2 = mColour2.rgb(fp);
    
    float fc = mColourNoise.value(x, y, z);
    int rgb = Col24.interp(rgb1, rgb2, fc);
    
    return rgb;
    
  } // SphericalSkin.colour()

} // class FieryOrangeSkin
