/**
 *  RoughGreySkin.java
 *  Copyright Simon Hern 2009
 *  Contact: dishmoth@yahoo.co.uk, www.dishmoth.com
 */

package Space.planets;

import Space.patterns.*;

// a texture for a planet
public class RoughGreySkin implements SphericalSkin {

  // noise functions
  private Function mCellPatternNoise,
                   mFractalPatternNoise,
                   mColourNoise;

  // colour ranges
  private ColourMap mColour1,
                    mColour2;
  
  // constructor
  public RoughGreySkin() {
    
    mCellPatternNoise = new Normalize(0.0f, 0.08f,
                          new VoronoiSphere(777, 5000));
    
    mFractalPatternNoise = new Scale(5.0f,
                             new Turbulence(
                               new Perlin3D(66), 3));
    
    mColourNoise = new Normalize(-0.5f, +0.5f,
                     new Scale(4.0f,
                       new FractalSum(
                         new Perlin3D(293), 2)));
    
    float x[] = {  0.0f,
                  +0.5f };
    int rgb1[] = { Col24.rgb(1.00f, 0.9f, 0.70f),
                   Col24.rgb(0.0f,  0.0f, 0.0f) },
        rgb2[] = { Col24.rgb(1.00f, 0.9f, 0.67f),
                   Col24.rgb(0.1f,  0.0f, 0.0f) };
    mColour1 = new ColourMap(x, rgb1);
    mColour2 = new ColourMap(x, rgb2);

  } // constructor
  
  // return a 24-bit RGB colour for a point on the unit sphere
  @Override
  public int colour(float x, float y, float z) {

    float f = 2.0f*mCellPatternNoise.value(x, y, z)
            + 0.2f*mFractalPatternNoise.value(x, y, z);
    
    int rgb1 = mColour1.rgb(f),
        rgb2 = mColour2.rgb(f);

    float fc = mColourNoise.value(x, y, z);
    int rgb = Col24.interp(rgb1, rgb2, fc);
    
    return rgb;
    
  } // SphericalSkin.colour()

} // class RoughGreySkin
